from pathlib import Path

from fire import Fire
from typing import Type, Iterable

from good_smell import LintSmell, implemented_smells, SmellWarning


def print_smell_warnings(path: str):
    """Prints any warning messages about smells"""
    print(
        "\n".join(
            warning.warning_string()
            for warning in smell_warnings(Path(path).read_text(), path)
        )
    )


def smell_warnings(source: str, path: str='') -> Iterable[SmellWarning]:
    for smell in implemented_smells:
        yield from smell.from_source(
            source_code=source, path=str(path)
        ).check_for_smell()


def print_fixed_smell(path: str, starting_line: int = 0, end_line: int = None):
    """Prints a fixed version of `source`"""
    pathlib_path = Path(path)
    source = pathlib_path.read_text()
    print(fix_smell(source, starting_line, end_line))


def fix_smell(
    source: str, starting_line: int = 0, end_line: int = None, path: str = None
) -> str:
    """Returns a fixed version of `source`"""
    smell: Type[LintSmell]
    for smell in implemented_smells:
        source = smell.from_source(
            source, starting_line, end_line, path=path
        ).fix_smell()
    return source


def main():
    Fire({"fix": print_fixed_smell, "warn": print_smell_warnings})


if __name__ == "__main__":
    main()



import ast
import astor
from good_smell import SmellWarning, LintSmell
from typing import Type, List
import abc


class LoggingTransformer(ast.NodeTransformer, abc.ABC):
    """A subclass of transformer that logs the nodes it transforms"""

    def __init__(self):
        self.transformed_nodes = list()

    @abc.abstractmethod
    def is_smelly(self, node: ast.AST) -> bool:
        """Checks if the given `node` should be transformed"""

    def visit(self, node: ast.AST):
        if not self.is_smelly(node):
            return self.generic_visit(node)
        self.transformed_nodes.append(node)
        return super().visit(node)


class AstSmell(LintSmell):
    def check_for_smell(self) -> List[SmellWarning]:
        """Return a list of all occuring smells of this smell class"""
        transformer = self.transformer_class()
        transformer.visit(self.tree)
        node: ast.stmt
        return [
            SmellWarning(
                msg=self.warning_message,
                row=node.lineno,
                col=node.col_offset,
                path=self.path,
                symbol=self.symbol
            )
            for node in transformer.transformed_nodes
        ]

    def fix_smell(self) -> str:
        """Return a fixed version of the code without the code smell"""
        return astor.to_source(self.transformer_class().visit(self.tree))

    @abc.abstractmethod
    def transformer_class(self) -> Type[LoggingTransformer]:
        """The class for the transformer used to create"""



import ast
import math
from good_smell import implemented_smells, SmellWarning
from typing import Tuple, Generator


class GoodSmellFlake8:
    """Entry point good smell to be used as a flake8 plugin"""

    name = "good-smell"
    version = "0.1"

    def __init__(self, tree: ast.AST, filename: str):
        """"http://flake8.pycqa.org/en/latest/plugin-development/plugin-parameters.html"""
        self.tree = tree
        self.filename = filename

    def run(self) -> Generator[Tuple[int, int, str, str], None, None]:
        for num, smell in enumerate(implemented_smells):
            warnings = smell(tree=self.tree, path=self.filename).check_for_smell()
            warning: SmellWarning
            yield from (
                (
                    warning.row,
                    warning.col,
                    f"SML{str(num).zfill(3)} {warning.msg}",
                    "GoodSmell",
                )
                for warning in warnings
            )



from .smell_warning import SmellWarning
from .lint_smell import LintSmell
from .ast_smell import AstSmell, LoggingTransformer
from .smells import implemented_smells
from . import smells  # Allow importing good_smell.smells
from .flake8_ext import GoodSmellFlake8
from .main import fix_smell, print_fixed_smell, main, smell_warnings



import abc
import ast
import os
from typing import List, Optional
from good_smell import SmellWarning


class LintSmell(abc.ABC):
    """Abstract Base class to represent the sniffing instructions for the linter"""

    def __init__(self, path: Optional[str] = None, tree: Optional[ast.AST] = None):
        self.tree = tree
        self.path = path

    @classmethod
    def from_source(
        cls,
        source_code: Optional[str] = None,
        start_line: Optional[int] = 0,
        end_line: Optional[int] = None,
        path: Optional[str] = None,
    ) -> "LintSmell":
        start_line = start_line
        end_line = end_line or len(source_code.splitlines())
        source_code = os.linesep.join(source_code.splitlines()[start_line:end_line])
        return cls(path=path, tree=ast.parse(source_code))

    @abc.abstractmethod
    def check_for_smell(self) -> List[SmellWarning]:
        """Return a list of all occuring smells of this smell class"""

    @abc.abstractmethod
    def fix_smell(self) -> str:
        """Return a fixed version of the code without the code smell"""

    @property
    @abc.abstractmethod
    def symbol(self) -> str:
        """The symbolic name for the smell"""

    @property
    @abc.abstractmethod
    def warning_message(self) -> str:
        """The symbolic name for the smell"""



from good_smell import AstSmell, LoggingTransformer
import ast


class NestedFor(AstSmell):
    """Checks for adjacent nested fors and replaces them with itertools.product"""

    @property
    def transformer_class(self):
        return NestedForTransformer

    @property
    def warning_message(self):
        return "Consider using itertools.product instead of a nested for"

    @property
    def code(self):
        return "SML002"

    @property
    def symbol(self):
        return "nested-for"


class NestedForTransformer(LoggingTransformer):
    """NodeTransformer that goes visits all the nested `for`s and replaces them
    with itertools.product"""

    def visit_For(self, node: ast.For) -> ast.For:
        inner_for: ast.For = node.body[0]
        import_itertools = ast_node("import itertools")
        itertools_product = ast_node("itertools.product").value
        new_for = ast.For(
            target=ast.Tuple(elts=[node.target, inner_for.target]),
            iter=ast.Call(
                func=itertools_product, args=[node.iter, inner_for.iter], keywords=[]
            ),
            body=inner_for.body,
            orelse=node.orelse,
        )
        new_for = ast.fix_missing_locations(new_for)
        return [ast.copy_location(import_itertools, node), new_for]

    @staticmethod
    def is_smelly(node: ast.AST):
        """Check if the node is only a nested for"""
        try:
            return (
                isinstance(node, ast.For)
                and isinstance(node.body[0], ast.For)
                and len(node.body) == 1
            )
        except AttributeError:
            return False


def ast_node(expr: str) -> ast.AST:
    """Helper function to parse a string denoting an expression into an AST node"""
    # ast.parse returns "Module(body=[Node])"
    return ast.parse(expr).body[0]



import dataclasses

FLAKE8_FORMAT = "{path}:{row}:{col} {symbol} {msg}"
PYLINT_FORMAT = "{path}:{line}:{column}: {msg} ({symbol})"


@dataclasses.dataclass(frozen=True)  # pylint: disable=too-few-public-methods
class SmellWarning:
    """Class to represent a warning message about a smell"""

    row: int
    col: int
    path: str
    msg: str
    symbol: str

    def warning_string(self, formatter: str = PYLINT_FORMAT):
        return formatter.format(**dataclasses.asdict(self))



import sys
from pathlib import Path

from setuptools import setup, find_packages

needs_pytest = {"pytest", "test", "ptr"}.intersection(sys.argv)
pytest_runner = ["pytest-runner==4.2"] if needs_pytest else []
long_description = Path("README.md").read_text()

needs_pytest = {"pytest", "test", "ptr"}.intersection(sys.argv)
pytest_runner = ["pytest-runner==4.2"] if needs_pytest else []
dataclasses = (
    [] if sys.version_info < (3, 7) else []
)  # Backport of dataclasses from py3.7

tests_require = [
    "pytest==4.0.1",
    "mccabe==0.6.1",
    "pytest-mccabe==0.1",
    "autopep8==1.4.3",
]
setup(
    name="good_smell",
    version="0.7",
    packages=find_packages(exclude=("tests",)),
    setup_requires=[] + pytest_runner,
    install_requires=[
        "fire==0.1.3",
        "astor==0.7.1",
        "astpretty==1.4.0",
        "flake8 >= 3.0.0",
        'dataclasses==0.6;python_version<"3.7"',
    ],
    author="Tomer Keren",
    author_email="tomer.keren.dev@gmail.com",
    description="A linter/refactoring tool to make your code smell better!",
    long_description=long_description,
    license="BSD",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Framework :: Flake8",
        "Environment :: Console",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: BSD License",
    ],
    url="https://github.com/Tadaboody/good_smell",
    tests_require=tests_require,
    entry_points={
        "console_scripts": ["good_smell=good_smell:main"],
        "flake8.extension": "SML=good_smell:GoodSmellFlake8",
    },
    extras_require={"dev": tests_require},
)



import ast

from good_smell import AstSmell, LoggingTransformer
from typing import Union


class RangeLenSmell(AstSmell):
    @property
    def transformer_class(self):
        return EnumerateFixer

    @property
    def symbol(self):
        return "range-len"

    @property
    def warning_message(self) -> str:
        return "Instead of using a c-style for loop, try using enumerate!"


class AssignDeleter(ast.NodeTransformer):
    def __init__(self, seq: ast.Name, target: ast.Name):
        self.id = target
        self.seq = seq
        self.elem_target = None

    def visit_Assign(self, node: ast.Assign):
        """Deletes a node if it assigning using the for target"""
        if (
            isinstance(node.value, ast.Subscript)
            and node.value.slice.value.id == self.id.id
            and node.value.value.id == self.seq.id
        ):
            self.elem_target = node.targets[0]
            return None
        return node


class EnumerateFixer(LoggingTransformer):
    def visit_For(self, node: ast.For) -> Union[bool, ast.For]:
        enumerate_node = ast.Name(id="enumerate", ctx=ast.Load())
        node_iterable = node.iter.args[0].args[0]
        original_target = node.target
        deleter = AssignDeleter(target=original_target, seq=node_iterable)
        new_body = deleter.visit(node).body or [ast.Pass()]
        elm_target = deleter.elem_target or ast.Name(id="elm", ctx=ast.Store())
        # for (original_target,elm_target) in enumerate(node_iterable):
        new_node = ast.For(
            target=ast.Tuple(elts=[original_target, elm_target], ctx=ast.Store()),
            iter=ast.Call(func=enumerate_node, args=[node_iterable], keywords=[]),
            body=new_body,
            orelse=node.orelse,
        )
        new_node = ast.fix_missing_locations(ast.copy_location(new_node, node))
        return new_node

    @staticmethod
    def is_smelly(node: ast.For):
        try:
            return node.iter.func.id == "range" and node.iter.args[0].func.id == "len"
        except AttributeError:
            return False
